@@ -17,6 +17,7 @@ import uuid
 
 # local libraries
 from nion.data import Calibration
+from nion.instrumentation import DriftTracker
 from nion.instrumentation import HardwareSource
 from nion.swift.model import DocumentModel
 from nion.swift.model import Graphics
@@ -29,18 +30,20 @@ from nion.utils import Registry
 if typing.TYPE_CHECKING:
     from nion.swift.model import DataItem
     from nion.swift.model import DisplayItem
-    from . import camera_base
-    from . import scan_base
+    from nion.instrumentation import camera_base
+    from nion.instrumentation import scan_base
 
 _VectorType = typing.Tuple[typing.Tuple[float, float], typing.Tuple[float, float]]
 
 _ = gettext.gettext
 
 
-class PMTType(enum.IntEnum):
+class PMTTypeEnum(enum.IntEnum):
     DF = 0
     BF = 1
 
+PMTType = typing.Union[PMTTypeEnum, int]
+
 
 class SubscanState(enum.Enum):
     INVALID = -1
@@ -70,14 +73,29 @@ class DriftCorrectionSettings:
 AxisType = typing.Tuple[str, str]
 
 
-class AxisDescription:
-    def __init__(self, axis_id: str, axis1: str, axis2: str, display_name: str):
-        self.axis_id = axis_id
-        self.axis_type = (axis1, axis2)
-        self.display_name = display_name
+class AxisDescription(typing.Protocol):
 
-    def __str__(self) -> str:
-        return self.display_name
+    @property
+    def axis_id(self) -> str:
+        """Read-only property for the (ideally unique) identifier of this axis.
+
+        """
+        raise NotImplementedError()
+
+    @property
+    def axis_type(self) -> typing.Tuple[str, str]:
+        """Read-only property for the co-ordinate names of this axis.
+
+        Note: This might be removed in a future release
+        """
+        raise NotImplementedError()
+
+    @property
+    def display_name(self) -> str:
+        """Read-only property for the name of this axis as it appears in the UI
+
+        """
+        raise NotImplementedError()
 
 
 class ScanContext:
@@ -191,6 +209,7 @@ class STEMController(Observable.Observable):
         self.__ronchigram_camera: typing.Optional[camera_base.CameraHardwareSource] = None
         self.__eels_camera: typing.Optional[camera_base.CameraHardwareSource] = None
         self.__scan_controller: typing.Optional[scan_base.ScanHardwareSource] = None
+        self.__drift_tracker: typing.Optional[DriftTracker.DriftTracker] = None
 
     def close(self) -> None:
         self.__scan_context_channel_map = typing.cast(typing.Any, None)
@@ -245,6 +264,13 @@ class STEMController(Observable.Observable):
     def set_scan_controller(self, scan_controller: typing.Optional[HardwareSource.HardwareSource]) -> None:
         self.__scan_controller = typing.cast(typing.Optional["scan_base.ScanHardwareSource"], scan_controller)
 
+    @property
+    def drift_tracker(self) -> typing.Optional[DriftTracker.DriftTracker]:
+        if self.__drift_tracker:
+            return self.__drift_tracker
+        return typing.cast(typing.Optional[DriftTracker.DriftTracker],
+                           Registry.get_component("drift_tracker"))
+
     # end configuration methods
 
     def _enter_scanning_state(self) -> None:
@@ -502,6 +528,17 @@ class STEMController(Observable.Observable):
         """
         raise NotImplementedError()
 
+    def axis_transform_point(self, point: Geometry.FloatPoint, from_axis: AxisDescription, to_axis: AxisDescription) -> Geometry.FloatPoint:
+        """
+        Convert the vector "value" from "from_axis" to "to_axis".
+
+        Existing axis descriptions can be retrieved via:
+        `STEMController.axis_descriptions`
+
+        Raises `ValueError` if an invalid axis is passed as "from_axis" or "to_axis".
+        """
+        raise NotImplementedError()
+
     # end instrument API
 
     # required functions (templates). subclasses should override.
@@ -648,7 +685,7 @@ class GraphicSetController:
             del display_about_to_be_removed_listener
         self.__graphic_trackers = list()
         for graphic in graphics:
-            graphic.display_item.remove_graphic(graphic)
+            graphic.display_item.remove_graphic(graphic).close()
 
     def __remove_one_graphic(self, graphic_to_remove: Graphics.Graphic) -> None:
         graphic_trackers = list()
@@ -744,7 +781,7 @@ class DisplayItemListModel(Observable.Observable):
         for display_item in display_items:
             for graphic in display_item.graphics:
                 if graphic.graphic_id == graphic_id:
-                    display_item.remove_graphic(graphic)
+                    display_item.remove_graphic(graphic).close()
 
 def make_scan_display_item_list_model(document_model: DocumentModel.DocumentModel, stem_controller: STEMController) -> DisplayItemListModel:
     def is_scan_context_display_item(display_item: DisplayItem.DisplayItem) -> bool:
@@ -1193,7 +1230,7 @@ class DriftView(EventLoopMonitor):
             self.__graphic_about_to_be_removed_listener.close()
             self.__graphic_about_to_be_removed_listener = None
         if self.__graphic_display_item and self.__graphic:
-            self.__graphic_display_item.remove_graphic(self.__graphic)
+            self.__graphic_display_item.remove_graphic(self.__graphic).close()
             self.__graphic_display_item = None
         self.__graphic = None
 
