@@ -5,6 +5,7 @@ import unittest
 from nion.data import Calibration
 from nion.data import DataAndMetadata
 from nion.instrumentation import Acquisition
+from nion.swift.test import TestContext
 from nion.utils import Geometry
 
 
@@ -19,7 +20,7 @@ class ScanDataStream(Acquisition.DataStream):
 
     partial_length is the size of each chunk of data (number of samples) to send at once.
     """
-    def __init__(self, frame_count: int, scan_shape: Acquisition.ShapeType, channels: typing.Sequence[Acquisition.Channel], partial_length: int):
+    def __init__(self, frame_count: int, scan_shape: Acquisition.ShapeType, channels: typing.Sequence[Acquisition.Channel], partial_length: int, *, error_after: typing.Optional[int] = None):
         super().__init__(frame_count * numpy.product(scan_shape, dtype=numpy.int64))
         # frame counts are used for allocating and returning test data
         self.__frame_count = frame_count
@@ -34,6 +35,7 @@ class ScanDataStream(Acquisition.DataStream):
         self.__partial_index = 0
         self.data = {channel: numpy.random.randn(self.__frame_count, self.__scan_length) for channel in channels}
         self.prepare_count = 0
+        self.__error_after = error_after
 
     @property
     def channels(self) -> typing.Tuple[Acquisition.Channel, ...]:
@@ -56,11 +58,14 @@ class ScanDataStream(Acquisition.DataStream):
         source_data_slice = (slice(start_index, stop_index),)
         state = Acquisition.DataStreamStateEnum.PARTIAL if stop_index < self.__scan_length else Acquisition.DataStreamStateEnum.COMPLETE
         for channel in self.channels:
+            if self.__error_after is not None:
+                if self.__error_after == 0:
+                    new_count = 0  # this will trigger an exception in send data
+                self.__error_after -= 1
             data_stream_event = Acquisition.DataStreamEventArgs(self, channel, data_metadata,
                                                                 self.data[channel][self.__frame_index],
-                                                                new_count, source_data_slice, state)
+                                                                new_count, source_data_slice, Acquisition.DataStreamStateEnum.COMPLETE)
             self.fire_data_available(data_stream_event)
-            self._sequence_next(channel, new_count)
         # update indexes
         if state == Acquisition.DataStreamStateEnum.COMPLETE:
             self.__partial_index = 0
@@ -80,7 +85,7 @@ class SingleFrameDataStream(Acquisition.DataStream):
 
     partial_height is the size of each chunk of data (number of samples) to send at once.
     """
-    def __init__(self, frame_count: int, frame_shape: Acquisition.ShapeType, channel: Acquisition.Channel, partial_height: typing.Optional[int] = None):
+    def __init__(self, frame_count: int, frame_shape: Acquisition.ShapeType, channel: Acquisition.Channel, partial_height: typing.Optional[int] = None, error_after: typing.Optional[int] = None):
         super().__init__(frame_count)
         assert len(frame_shape) == 2
         # frame counts are used for allocating and returning test data
@@ -93,6 +98,7 @@ class SingleFrameDataStream(Acquisition.DataStream):
         self.__partial_height = partial_height or frame_shape[0]
         self.__partial_index = 0
         self.data = numpy.random.randn(self.__frame_count, *self.__frame_shape)
+        self.__error_after = error_after
 
     @property
     def channels(self) -> typing.Tuple[Acquisition.Channel, ...]:
@@ -124,7 +130,11 @@ class SingleFrameDataStream(Acquisition.DataStream):
         else:
             self.__partial_index = 0
             self.__frame_index += 1
-            self._sequence_next(self.__channel)
+        if self.__error_after is not None:
+            if self.__error_after == 0:
+                raise Exception()
+            self.__error_after -= 1
+
 
 
 class MultiFrameDataStream(Acquisition.DataStream):
@@ -193,7 +203,6 @@ class MultiFrameDataStream(Acquisition.DataStream):
                                                             source_data_slice, state)
         self.fire_data_available(data_stream_event)
         self.__frame_index += count
-        self._sequence_next(self.__channel, count)
 
 
 class RectangleMask(Acquisition.MaskLike):
@@ -224,10 +233,10 @@ class OrMask(Acquisition.MaskLike):
 class TestAcquisitionClass(unittest.TestCase):
 
     def setUp(self):
-        pass
+        TestContext.begin_leaks()
 
     def tearDown(self):
-        pass
+        TestContext.end_leaks(self)
 
     def test_unravel_flat_slice(self):
         test_cases = [
@@ -295,10 +304,8 @@ class TestAcquisitionClass(unittest.TestCase):
         collection_shape = (4, )
         channel = Acquisition.Channel("0")
         data_stream = SingleFrameDataStream(numpy.product(collection_shape), (2, 2), channel, partial_height=1)
-        collector_slices = [tuple(Acquisition.index_slice(n) for n in i) for i in numpy.ndindex(*collection_shape)]
-        collector = Acquisition.CollectedDataStream(data_stream, collection_shape,
-                                                    [Calibration.Calibration()],#, Calibration.Calibration()],
-                                                    collector_slices)
+        collectors = [Acquisition.CollectedDataStream(data_stream, (1,), [Calibration.Calibration()]) for i in numpy.ndindex(*collection_shape)]
+        collector = Acquisition.StackedDataStream(collectors)
         maker = Acquisition.FramedDataStream(collector)
         with maker.ref():
             Acquisition.acquire(maker)
@@ -649,8 +656,10 @@ class TestAcquisitionClass(unittest.TestCase):
         scan_data_stream = ScanDataStream(sequence_len, scan_shape, [channel0, channel1], scan_shape[1])
         camera_data_stream = SingleFrameDataStream(sequence_len * numpy.product(scan_shape), (2, 2), channel2)
         combined_data_stream = Acquisition.CombinedDataStream([scan_data_stream, camera_data_stream])
-        scan_slices = [[slice(0, 4), slice(0, 8)], [slice(4, 8), slice(0, 8)]]
-        collector = Acquisition.CollectedDataStream(combined_data_stream, scan_shape, [Calibration.Calibration(), Calibration.Calibration()], scan_slices)
+        collector = Acquisition.StackedDataStream([
+            Acquisition.CollectedDataStream(combined_data_stream, (4, 8), [Calibration.Calibration(), Calibration.Calibration()]),
+            Acquisition.CollectedDataStream(combined_data_stream, (4, 8), [Calibration.Calibration(), Calibration.Calibration()]),
+            ])
         sequencer = Acquisition.SequenceDataStream(collector, sequence_len)
         maker = Acquisition.FramedDataStream(sequencer)
         with maker.ref():
@@ -678,8 +687,7 @@ class TestAcquisitionClass(unittest.TestCase):
         camera_data_stream = SingleFrameDataStream(sequence_len * numpy.product(scan_shape), (2, 2), channel2)
         combined_data_stream = Acquisition.CombinedDataStream([scan_data_stream, camera_data_stream])
         collector = Acquisition.CollectedDataStream(combined_data_stream, scan_shape, [Calibration.Calibration(), Calibration.Calibration()])
-        slices = tuple((slice(i, i + 1),) for i in range(sequence_len))
-        sequencer = Acquisition.SequenceDataStream(collector, sequence_len, sub_slices=slices)
+        sequencer = Acquisition.StackedDataStream([Acquisition.SequenceDataStream(collector, 1) for i in range(sequence_len)])
         maker = Acquisition.FramedDataStream(sequencer)
         with maker.ref():
             Acquisition.acquire(maker)
@@ -705,8 +713,10 @@ class TestAcquisitionClass(unittest.TestCase):
         scan_data_stream = ScanDataStream(sequence_len, scan_shape, [channel0, channel1], scan_shape[1])
         camera_data_stream = SingleFrameDataStream(sequence_len * numpy.product(scan_shape), (2, 2), channel2)
         combined_data_stream = Acquisition.CombinedDataStream([scan_data_stream, camera_data_stream])
-        scan_slices = [[slice(0, 4), slice(0, 8)], [slice(4, 8), slice(0, 8)]]
-        collector = Acquisition.CollectedDataStream(combined_data_stream, scan_shape, [Calibration.Calibration(), Calibration.Calibration()], scan_slices)
+        collector = Acquisition.StackedDataStream([
+            Acquisition.CollectedDataStream(combined_data_stream, (4, 8), [Calibration.Calibration(), Calibration.Calibration()]),
+            Acquisition.CollectedDataStream(combined_data_stream, (4, 8), [Calibration.Calibration(), Calibration.Calibration()]),
+            ])
         sequencer = Acquisition.SequenceDataStream(collector, sequence_len)
         accumulator = Acquisition.AccumulatedDataStream(sequencer)
         maker = Acquisition.FramedDataStream(accumulator)
@@ -723,6 +733,19 @@ class TestAcquisitionClass(unittest.TestCase):
             self.assertEqual(DataAndMetadata.DataDescriptor(False, 2, 2), maker.get_data(channel2).data_descriptor)
             self.assertEqual(sequence_len * 2, scan_data_stream.prepare_count)
 
+    def test_sequence_of_stacked_collections(self):
+        channel = Acquisition.Channel("Cam")
+        camera_data_stream = SingleFrameDataStream(32, (2, 2), channel)
+        section1 = Acquisition.CollectedDataStream(camera_data_stream, (2, 4), [Calibration.Calibration(), Calibration.Calibration()])
+        section2 = Acquisition.CollectedDataStream(camera_data_stream, (2, 4), [Calibration.Calibration(), Calibration.Calibration()])
+        sections = Acquisition.StackedDataStream((section1, section2))
+        maker = Acquisition.FramedDataStream(Acquisition.SequenceDataStream(sections, 2))
+        with maker.ref():
+            Acquisition.acquire(maker)
+            self.assertFalse(maker.is_error)
+            maker_data = maker.get_data(channel)
+            self.assertTrue(numpy.array_equal(numpy.reshape(camera_data_stream.data, maker_data.data_shape), maker_data.data))
+
     def test_scan_as_collection_sequential(self):
         channel = Acquisition.Channel("Cam")
         sequence_len1 = 4
@@ -780,3 +803,38 @@ class TestAcquisitionClass(unittest.TestCase):
         self.assertEqual(1, action._s)
         self.assertEqual(4, action._p)
         self.assertEqual(1, action._f)
+
+    def test_error_during_send_next(self):
+        sequence_len = 4
+        channel = Acquisition.Channel("0")
+        data_stream = SingleFrameDataStream(sequence_len, (2, 2), channel, error_after=0)
+        sequencer = Acquisition.SequenceDataStream(data_stream, sequence_len)
+        maker = Acquisition.FramedDataStream(sequencer)
+        with maker.ref():
+            had_error = False
+
+            def handle_error(e: Exception) -> None:
+                nonlocal had_error
+                had_error = True
+
+            Acquisition.acquire(maker, error_handler=handle_error)
+            self.assertTrue(had_error)
+            self.assertTrue(maker.is_error)
+
+    def test_error_during_data_available(self):
+        scan_shape = (8, 8)
+        channel = Acquisition.Channel("0")
+        data_stream = ScanDataStream(1, scan_shape, [channel], scan_shape[1], error_after=1)
+        collector = Acquisition.CollectedDataStream(data_stream, scan_shape, [Calibration.Calibration(), Calibration.Calibration()])
+        maker = Acquisition.FramedDataStream(collector)
+        with maker.ref():
+            had_error = False
+
+            def handle_error(e: Exception) -> None:
+                nonlocal had_error
+                had_error = True
+
+            Acquisition.acquire(maker, error_handler=handle_error)
+            self.assertTrue(had_error)
+            self.assertTrue(maker.is_error)
+            self.assertAlmostEqual(1/8, collector.progress)
