@@ -6,6 +6,8 @@ import time
 import typing
 import unittest
 import uuid
+import tempfile
+import pathlib
 
 from nion.data import DataAndMetadata
 from nion.swift import Application
@@ -20,6 +22,7 @@ from nion.instrumentation import camera_base
 from nion.instrumentation import DataChannel
 from nion.instrumentation import DriftTracker
 from nion.instrumentation import scan_base
+from nion.instrumentation import AcquisitionPreferences
 from nion.instrumentation.test import AcquisitionTestContext
 from nionswift_plugin.nion_instrumentation_ui import ScanAcquisition
 
@@ -433,6 +436,27 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
 
             # self.assertEqual("Ned Flanders", data_item.session_metadata["microscopist"])
 
+    def test_partial_acquisition_while_view_mode_running(self):
+        with self.__test_context(is_eels=True) as test_context:
+            scan_hardware_source = test_context.scan_hardware_source
+            camera_hardware_source = test_context.camera_hardware_source
+            scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
+            scan_frame_parameters.scan_id = uuid.uuid4()
+            scan_frame_parameters.size = Geometry.IntSize(6, 6)
+            camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
+            camera_frame_parameters.processing = "sum_project"
+            camera_hardware_source.start_playing(camera_frame_parameters, sync_timeout=3.0)
+            try:
+                camera_hardware_source.get_next_xdatas_to_finish(camera_frame_parameters.exposure * 10)
+                section_height = 5
+                scan_hardware_source.grab_synchronized(scan_frame_parameters=scan_frame_parameters,
+                                                       camera=camera_hardware_source,
+                                                       camera_frame_parameters=camera_frame_parameters,
+                                                       section_height=section_height)
+                camera_hardware_source.get_next_xdatas_to_finish(camera_frame_parameters.exposure * 10)
+            finally:
+                camera_hardware_source.stop_playing(sync_timeout=3.0)
+
     def test_partial_acquisition_has_proper_metadata(self):
 
         class TestDataChannel(DataChannel.DataItemDataChannel):
@@ -518,6 +542,61 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
                 # self.assertIn("rotation", scan_metadata)
                 # self.assertIn("scan_id", scan_metadata)
 
+    def test_partial_acquisition_has_proper_metadata_in_associated_view(self):
+        # start an SI. use data channel to record the spectrum image item used for the view during the
+        # acquisition and also call periodic. probably better if grab_synchronized is modified to support this.
+
+        class TestDataChannel(DataChannel.DataItemDataChannel):
+            def __init__(self, document_controller, data_item, channel_names: typing.Mapping[Acquisition.Channel, str]):
+                super().__init__(document_controller.document_model, "test", channel_names)
+                self.__document_controller = document_controller
+                self.__data_item = data_item
+                self.updates = list()
+
+            def update_data(self, channel: Acquisition.Channel, source_data: numpy.ndarray,
+                            source_slice: Acquisition.SliceType, dest_slice: slice,
+                            data_metadata: DataAndMetadata.DataMetadata) -> None:
+                super().update_data(channel, source_data, source_slice, dest_slice, data_metadata)
+                self.updates.append(self.__data_item.dimensional_calibrations)
+                self.__document_controller.periodic()
+
+        with self.__test_context(is_eels=True) as test_context:
+            scan_hardware_source = test_context.scan_hardware_source
+            camera_hardware_source = test_context.camera_hardware_source
+            scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
+            scan_frame_parameters.scan_id = uuid.uuid4()
+            scan_frame_parameters.size = Geometry.IntSize(2, 4)
+            camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
+            camera_frame_parameters.processing = "sum_project"
+
+            # start the camera playing; ensure we have a data item
+            camera_hardware_source.start_playing(sync_timeout=3.0)
+            try:
+                camera_hardware_source.get_next_xdatas_to_finish(camera_frame_parameters.exposure * 10)
+                test_context.document_controller.periodic()
+                data_item = test_context.document_model.data_items[0]
+                self.assertEqual("eV", data_item.dimensional_calibrations[-1].units)
+
+                data_channel = TestDataChannel(test_context.document_controller, data_item,
+                                               {Acquisition.Channel(scan_hardware_source.hardware_source_id, "0"): "HAADF",
+                                                Acquisition.Channel(camera_hardware_source.hardware_source_id): "test"})
+
+                section_height = 2
+                scan_hardware_source.grab_synchronized(data_channel=data_channel,
+                                                       scan_frame_parameters=scan_frame_parameters,
+                                                       camera=camera_hardware_source,
+                                                       camera_frame_parameters=camera_frame_parameters,
+                                                       section_height=section_height)
+
+                camera_hardware_source.get_next_xdatas_to_start(camera_frame_parameters.exposure * 10)
+                self.assertEqual("eV", data_item.dimensional_calibrations[-1].units)
+            finally:
+                camera_hardware_source.stop_playing(sync_timeout=3.0)
+                test_context.document_controller.periodic()
+
+            for update in data_channel.updates:
+                self.assertEqual("eV", update[-1].units)
+
     def test_grab_synchronized_basic_eels_with_drift_correction(self):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
@@ -700,6 +779,51 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             self.assertTrue(1.9 < abs(last_delta_nm.width) < 2.1)
             self.assertTrue(abs(last_delta_nm.height) < 0.1)
 
+    def test_drift_corrector_uses_customized_drift_scan_frame_parameters(self):
+        # for this test, drift should be in a constant direction
+        with self.__test_context() as test_context:
+            with tempfile.TemporaryDirectory() as temp_dir:
+                file_path = pathlib.Path(temp_dir) / pathlib.Path("nion_acquisition_preferences.json")
+                AcquisitionPreferences.init_acquisition_preferences(file_path)
+                document_controller = test_context.document_controller
+                scan_hardware_source = test_context.scan_hardware_source
+                test_context.instrument.sample_index = 2  # use CTS sample, custom position chosen using view mode in Swift
+                drift_tracker = scan_hardware_source.drift_tracker
+                self._acquire_one(document_controller, scan_hardware_source)
+                scan_hardware_source.drift_channel_id = scan_hardware_source.data_channels[0].channel_id
+                scan_hardware_source.drift_region = Geometry.FloatRect.from_center_and_size(Geometry.FloatPoint(0.6554, 0.2932), Geometry.FloatSize(0.15, 0.15))
+                document_controller.periodic()
+                # Change the default parameters (64 px and 16 us) to something different
+                scan_customization = AcquisitionPreferences.acquisition_preferences.drift_scan_customization
+                # Use something that is unlikely to be used as a default value in the future
+                scan_customization.dwell_time_us = 17.3
+                scan_customization.scan_width_pixels = 111
+                scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
+                drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(drift_tracker, scan_hardware_source, scan_frame_parameters, 0)
+                self.assertEqual(0.0, drift_tracker.last_delta_nm.width)
+                self.assertEqual(0.0, drift_tracker.last_delta_nm.height)
+                drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
+                last_delta_nm = drift_tracker.last_delta_nm
+                dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
+                self.assertLess(dist_nm, 0.1)
+                stem_controller = test_context.instrument
+                stem_controller.SetValDeltaAndConfirm("CSH.x", 1e-9, 1.0, 1000)
+                drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
+                last_delta_nm = drift_tracker.last_delta_nm
+                dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
+                self.assertTrue(0.9 < dist_nm < 1.1)
+                self.assertTrue(0.9 < abs(last_delta_nm.width) < 1.1)
+                self.assertTrue(abs(last_delta_nm.height) < 0.1)
+                stem_controller.SetValDeltaAndConfirm("CSH.x", 2e-9, 1.0, 1000)
+                drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
+                last_delta_nm = drift_tracker.last_delta_nm
+                dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
+                self.assertTrue(1.9 < dist_nm < 2.1)
+                self.assertTrue(1.9 < abs(last_delta_nm.width) < 2.1)
+                self.assertTrue(abs(last_delta_nm.height) < 0.1)
+                self.assertSequenceEqual(drift_tracker._first_xdata.data_shape, (111, 111))
+                self.assertEqual(drift_tracker._first_xdata.metadata["hardware_source"]["pixel_time_us"], 17.3)
+
     def test_scan_acquisition_controller(self):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
